//////////////////////
// vim: fileencoding=utf-8:nomodified:nowrap:textwidth=0:foldmethod=marker:foldcolumn=4:ruler:showcmd:lcs=tab\:|- list:noexpandtab:nosmarttab:softtabstop=0:shiftwidth=0
// Generated 2018.03.19 02:04:19 from knowledge.txt
//////////////////////

#ifndef ENGINE_H
#define ENGINE_H

// for uint*_t
#include <stdint.h>
#include "knowledge.h"
#include "MyProgmemTools.h"
#include <Arduino.h>


#define BUS_LEN 16
typedef char T_Bus[BUS_LEN+1];	// Universal type for all pins and values (+1 for \0)
	// until critical, use char for symbolic values, compact later, if needed
enum E_VERBOSITY{	// {{{
	E_VERBOSITY_min,
	V_NONE=E_VERBOSITY_min,
	V_ERROR,
	V_MILESTONE,
	V_INFO,
	V_DEBUG,
	V_TRACE,
	V_ALL,
	E_VERBOSITY_max 	// the len of valid E_VERBOSITY
}; 	// }}}
// {{{ PHASES
#define P_INPUTS   0
#define P_PREPARE  1
#define P_EDGE     2
#define P_CLOCK    3
#define P_VALUES   4
#define P_3STATE   5
// }}}
#define TG_NONE 0
#define TG_LED 1
#define TG_SERIAL 2
#define TG_OLED 4
#define TG_ALL (TG_LED|TG_SERIAL|TG_OLED)
class C_decoder;	// forward, inner to C_Engine
class C_Engine {	// {{{
private:
//	GLOBAL/HW
T_Bus	pins;	// arduino pins for each position as byte - ff = NC/NU
//	UNIT to test
const S_Unit	*unit;
const S_Line	*line;	// actual unit & line
const S_Line	*max_line;	// actual unit & line
//	UNIT processing 	// T_Bus ' ' means empty/not defined yet/etc.
T_Bus	master_mask_line;	// Power, NC, etc. to skip (copyied to fetched_line too)
T_Bus	actual_line;	// plain decoded line (R)
T_Bus	fetched_line;	// all F's and X's to skip
T_Bus	test_case;	// fully decoded line for test (^vLH@aCe...)
T_Bus	test_line;	// test_case with actual values (^vLH^Ce) counters computed
//	INTERNALS
T_Bus	cached_line;	// what is actually set (^v01) - cache to avoid duplicite pinMode/digitalWrite
bool	counter_runs;	//
uint8_t	counter;	//
//	STATS
uint16_t unit_no;	// number of processed units
uint16_t unit_skipped;	// number of skipped units (due mask)
uint16_t unit_failed;	// number of failed units (due lines)
uint16_t line_no;	// number of actual line in unit
//	
C_decoder *decoder;
public:
//	SETTINGS
bool	do_extra_tests;	// continue with testing to end of unit? False: just identify; True: QA test
bool	filter_by_mask;	// test only what fits mask
bool	run_all_lines;	// test all lines in unit, even if failed DEBUG
uint8_t	log_targets;	// Serial,LEDs, OLED ..
uint8_t	verbosity;	// for logs
private:
uint8_t	num_pins;	// total number of pins including power
uint8_t	num_active_pins;	// number of active pins for testing
uint8_t	num_NC_pins;	// number of not connected pins for testing
T_Bus	tested_outputs;	// " O" if in fast test this was at least once driven
T_Bus	tested_inputs;	// " I" if in fast test change in this make some pins change somehow
T_Bus	result_line;	// temp for storing digitalRead for tests
/* FIXME to BUS_LEN */
uint16_t dO,kI;	// temporary subst for tested_intputs/tested_outputs
public:
	C_Engine() ;
	// constructor
	// 	LOGGING (true, if really logged) {{{
	bool do_log(uint8_t lvl,const char *msg);	// logging also line and unit and result
	bool trace(const char *msg);	// logging also line and unit and result
	bool debug(const char *msg);	// logging also line and unit and result
	bool info(const char *msg);	// logging also line and unit and result
	bool milestone(const char *msg);	// logging also line and unit and result
	bool error(const char *msg);	// logging also line and unit and result
	// 	// LOGGING with number
	bool do_log(uint8_t lvl,const char *msg,uint16_t number);	// logging also line and unit and result 
	bool trace(const char *msg,uint16_t number);	// logging also line and unit and result
	bool debug(const char *msg,uint16_t number);	// logging also line and unit and result
	bool info(const char *msg,uint16_t number);	// logging also line and unit and result
	bool milestone(const char *msg,uint16_t number);	// logging also line and unit and result
	bool error(const char *msg,uint16_t number);	// logging also line and unit and result
	// 	PROGMEM LOGGING (true, if really logged)
	bool do_log(uint8_t lvl,const __FlashStringHelper *msg);	// logging also line and unit and result
	bool trace(const __FlashStringHelper *msg);	// logging also line and unit and result
	bool debug(const __FlashStringHelper *msg);	// logging also line and unit and result
	bool info(const __FlashStringHelper *msg);	// logging also line and unit and result
	bool milestone(const __FlashStringHelper *msg);	// logging also line and unit and result
	bool error(const __FlashStringHelper *msg);	// logging also line and unit and result
	// 	// PROGMEM LOGGING with number
	bool do_log(uint8_t lvl,const __FlashStringHelper *msg,uint16_t number);	// logging also line and unit and result 
	bool trace(const __FlashStringHelper *msg,uint16_t number);	// logging also line and unit and result
	bool debug(const __FlashStringHelper *msg,uint16_t number);	// logging also line and unit and result
	bool info(const __FlashStringHelper *msg,uint16_t number);	// logging also line and unit and result
	bool milestone(const __FlashStringHelper *msg,uint16_t number);	// logging also line and unit and result
	bool error(const __FlashStringHelper *msg,uint16_t number);	// logging also line and unit and result
	// }}}
	// decoding lines and such - should return True everytime
	bool reset(uint8_t a_pins, uint16_t drivenOuts, uint16_t knownInputs);	// reset engine for new test batch
	bool unit_reset(const S_Unit *a_unit);	// reset engine for unit - false = do NOT test this unit
	bool decode_line(const S_Line *&a_line);	// decode and apply line (move a_line to next place) (a_line is source, engine T_Buses are target)
	bool run_current_line();	// run tests on current line
	bool run_current_line(uint8_t phase);	// run tests on current line in phase (like edge/clock/values/ ...)
	bool run_test();	// run test on current unit
	bool run_range(const S_Unit *start, const S_Unit *end);	// run tests on units in range
	void report_unit();	// report actual unit
	void report_line(const char *a_line);	// converts and prints line
	const __FlashStringHelper *get_unit_name(uint16_t u);
	const __FlashStringHelper *get_unit_comment(uint16_t u);
	const __FlashStringHelper *get_unit_name();
	const __FlashStringHelper *get_unit_comment();
	// Pin manipulation
	void SetPin(uint8_t pin, char value);	// Set pin to ^v10 (consult/set cached_line if needed)
	bool GetPin(uint8_t pin, char value);	// Return if pin really reads ^v

};	// }}}
class C_decoder {	// {{{
private:	// {{{
	uint8_t offset;	// offset in halfbytes for logger
	C_Engine *logger;	// logg here
// }}}
public:
	C_decoder(C_Engine * logger);
	uint8_t fill(const char *&start, char *target);	// fill target with decoded start, = num of chars
	uint8_t fillP(const char *&start, char *target);	// fill target with decoded start from PROGMEM, = num of chars
};	// }}}

#endif
